<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Tco00800Prospect {
    protected $environment;
    protected $entityManager;
    protected $tco00800ProspectService;
    
    public function __construct($environment, $entityManager, $tco00800ProspectService) {
        $this->environment                    = $environment;
        $this->entityManager                  = $entityManager;
        $this->tco00800ProspectService  = $tco00800ProspectService;
    }

    /**
     * Salva o registro de prospect da forma correta
     * fazendo verificações de já existência de 
     * registro e validade dos dados de endereço.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function saveProspect(DTO\Request $request, DTO\Response $response){
        $register = $request->getDataSet()->getRows()[0];

        $validityProspectSave = $this->validateProspectSave($register);
        
        if(!strlen($validityProspectSave)){
            $this->tco00800ProspectService->saveProspect($request, $response);
        } else {
            $response->setError(new Error($validityProspectSave, 1));
        }
            
    }
            
    /**
     * Valida os dados de salvamento de registro
     * de prospect. Verifica se o numero de inscrição
     * já existe no caso de novo registro ou registro
     * alterado, e se o email é válido em caso de novo
     * registro.
     * 
     * @param {DTO\Row} $register Registro de 
     *  prospect.
     */
    public function validateProspectSave($register){
        $isNew = $register->get('__is_new');
        
        $nrInsPros = $register->get('NRINSJURPROS');
        $cdProspect = $register->get('CDPROSPECT');

        $existProspect = false;

        $prospByIns = $this->tco00800ProspectService->existsProspect('I', $nrInsPros); 

        if($isNew){
            $existProspect = (bool)$prospByIns;
        } else {
            $prospByCode = $this->tco00800ProspectService->existsProspect('C', $cdProspect); 

            if((bool)$prospByIns){
                $existProspect = $prospByIns[0]['NRINSJURPROS'] != $prospByCode[0]['NRINSJURPROS'];
            }
        }

        if($register->has('DSEMAILPROS') && strlen($register->get('DSEMAILPROS')) && $isNew) {
                
            $isValidEmail = (bool)filter_var($register->get('DSEMAILPROS'), FILTER_VALIDATE_EMAIL);
            
        } else {
            $isValidEmail = true;
        }

        return $existProspect ? 'Impossível adicionar. O registro já existe.' :
             ($isValidEmail ? '' : 'Email inválido.');
    }

    /**
     * Remove o registro de prospect da forma correta,
     * fazendo verificações de existência de registros
     * filhos e excluindo os respectivos endereços.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function removeProspect(DTO\Request $request, DTO\Response $response){
        $cdProspect = $request->getRow()->get('CDPROSPECT');
        $prosDependencies = $this->tco00800ProspectService->getProspectDependencies($cdProspect);
        
        if(!sizeof($prosDependencies)){
            $this->tco00800ProspectService->removeProspect($request, $response);
        } else {
            $response->addDataSet(new DataSet('invalidRemove', $prosDependencies));
        }
    }

    /**
     * Salva inserção/alteração de registro de endereço
     * de prospect.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function saveAddress(DTO\Request $request, DTO\Response $response){
        $register = $request->getRow();
        $cdProspect = $register->get('CDPROSPECT');
        $idTpEnde = $register->get('IDTPENDEPROS');
        
        $existAddress = $this->tco00800ProspectService->existsEndeProspect($cdProspect, $idTpEnde);

        if(!$existAddress || !$register->get('__is_new')){
            $this->tco00800ProspectService->saveAddress($request, $response, $cdProspect, $idTpEnde);
        } else {
            $response->setError(new Error('O tipo de endereço selecionado já está cadastrado para o prospect.', 1));
        }
    }

    /**
     * Função responsável por remover endereços de
     * prospects.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function removeAddress(DTO\Request $request, DTO\Response $response){
        $this->tco00800ProspectService->removeAddress($request, $response);
    }

}